/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <ctype.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include "explode.h"

/* explodes a strings into substrings and returns the number of substrings */
int explode (EXPLDARGS)
{
   unsigned i, count, length;
   char *buf = (char *) malloc (strlen(str) + 1);
   
   if (!buf) {
      perror ("malloc");
      exit (1);
   }
   
   i = count = length = 0;
   for ( ; str[i]; i++) {
      if (str[i] != delimiter) 
         buf[length++] = str[i];
      else {
         buf[length] = '\0';
         explode_push_queue (buf, &(*tail)->next);
         (*tail) = (*tail)->next;
         
         count++;
         length = 0;
         strcpy (buf, "");
      }
   }
   
   /* check if there's a substring at the the end of the string */
   if (length && count) {
      buf[length] = '\0';
      explode_push_queue (buf, &(*tail)->next);
      (*tail) = (*tail)->next;
      count++;
   }
   
   free (buf);

   return count;
}

/* saves the substrings into a queue */
void explode_push_queue (EXPLDPUSHARGS)
{
   explode_t *newnode = (explode_t *) malloc (sizeof(explode_t));
   newnode->str = (char *) malloc (strlen(str) + 1);
   
   if (!newnode || !newnode->str) {
      perror ("malloc");
      exit (1);
   }
   
   newnode->next = NULL;
   strcpy (newnode->str, str);
   (*tail) = newnode;
}

/* deallocates the queue */
void explode_destroy_queue (EXPLDDESTROYARGS)
{
   explode_t *current = head, *next = NULL;
   
   while (current) {
      next = current->next;   
      free (current);
      current = next;
   }
}

/* converts a string to lower case */
char *strtolower (STRTOLOWERARGS)
{
   int i = 0;
   char *buf = (char *) malloc (strlen(str) + 1);
   
   if (!buf) {
      perror ("malloc");
      exit (1);
   }
   
   for ( ; str[i]; i++) 
      buf[i] = tolower(str[i]);
   
   buf[i] = '\0';
   
   return buf;
}
