/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# ifndef __explode_h_
# define __explode_h_

/* NOTICE: when exploding a line from the INDEX file
 * the elements returned by explode() are as follows:
 * ==================================================
 * (1) package name
 * (2) package port directory
 * (3) install prefix of the package
 * (4) description of the package
 * (5) pkg-descr file location
 * (6) maintainer
 * (7) category of the package
 * (8) build deps
 * (9) run deps
 * (10) website
 * (11) extract deps
 * (12) patch deps
 * (13) fetch deps
 * ==================================================
 * in the scan_index_file() function we take only 
 * these elements - (1), (4), (6) and (7), since we
 * need only this information.
 */

/* defines explode_t type */
typedef struct explode_struct {  /* stores information about the  */
   char *str;                    /* exploded string returned by  */
   struct explode_struct *next;  /* explode ()                    */
} explode_t;                     

/* define functions parameters as macros */
# define EXPLDARGS         const char delimiter, const char *str, explode_t **tail
# define EXPLDPUSHARGS     const char *str, explode_t **tail
# define EXPLDDESTROYARGS  explode_t *head
# define STRTOLOWERARGS    const char *str
   
int   explode (EXPLDARGS); /* explodes a string */
void  explode_push_queue (EXPLDPUSHARGS); /* saves a substring into a queue */
void  explode_destroy_queue (EXPLDDESTROYARGS); /* deallocates the queue */
char  *strtolower (STRTOLOWERARGS); /* converts a string to lower case */

# endif  /* ! ifndef __explode_h_ ! */
