/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <unistd.h>
# include <dirent.h>
# include <sys/stat.h>
# include <sys/types.h>
# include <sys/param.h>
# include "fnode.h"
# include "scan_directory.h"

/* checks for existence of a directory */
int dir_exists (DIREXISTSARGS)
{
   struct stat dir_stat;
   
   if (stat(dirname, &dir_stat) < 0) {
      perror ("stat");
      exit (1);
   }
   
   return S_ISDIR(dir_stat.st_mode);
}

/* recursively scans the specified directory for the package pattern */
int scan_directory (SCANDIRARGS)
{
   DIR *dir;
   struct dirent *entry;
   struct stat dir_stat;
   bool_t founddotdir;
   static unsigned count = 0;
   char buf[MAXPATHLEN + 1]; 
   
   if (!(dir = opendir(pathname))) {
      fprintf (stderr, "==>> Cannot open directory '%s'\n", pathname);
      exit (1);
   }
   if (chdir(pathname) == -1) {
      perror ("chdir");
      exit (1);
   }
   
   while ((entry = readdir (dir))) {
      if (stat(entry->d_name, &dir_stat) < 0) {
         perror ("stat");
         exit (1);
      }
      
      /* check if the entry is "." or ".." to avoid loops */
      founddotdir = !strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..");
      
      /* if it's a directory - search it too */ 
      if (S_ISDIR(dir_stat.st_mode) && !founddotdir) {
         /* save the current path */
         if (!getcwd(buf, MAXPATHLEN + 1)) {
            perror ("getcwd");
            exit (1);
         }
         
         scan_directory (entry->d_name, pattern, head);
         
         /* return to the last scanning dir */
         if (chdir(buf) == -1) {
            perror ("chdir");
            exit (1);
         }
      }
         
      
      /* make a record of the matched entry if it's a file */      
      if (strstr(entry->d_name, pattern) && \
         !founddotdir && !S_ISDIR(dir_stat.st_mode)) {        
            count++;
            if (!getcwd(buf, MAXPATHLEN + 1)) {
               perror ("getcwd");
               exit (1);
            }
            /* make a record of the matched pattern */
            fnode_push (head, entry->d_name, buf, NULL, NULL);
      }
   }
   
   (void)closedir (dir);
     
   return count;
}
