/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <unistd.h>
# include <sys/param.h>
# include "fnode.h"
# include "explode.h"
# include "scan_index_file.h"

/* used to check for existence of the INDEX file */
int file_exists (FILEEXISTSARGS)
{
   return (access(filename, F_OK) == 0);
}

/* fetches the INDEX file if it doesn't exist in the current dir */
void fetch_index_file (FETCHFILEARGS)
{
   char *ftppath, indexpath[MAXPATHLEN + 1];
         
   fprintf (stdout, "==>> NOTICE: INDEX file was not found in current dir, I'll fetch it..\n\n");
   ftppath = build_ftp_path ();
   
   snprintf (indexpath, sizeof(indexpath), "fetch %s/INDEX", ftppath);
   
   system (indexpath);
}

/* scans the INDEX file for a pattern */
int scan_index_file (SCANFILEARGS)
{
   int i;
   char *fname, *fpath, *desc, *maint;
   char bigbuf[100000]; /* a buffer big enough to store one line from INDEX */
   
   unsigned pattern_count, str_count;
   explode_t *expl_head, *expl_tail, *expl_current;
   FILE *fp;
   
   /* fetch the INDEX file it's not in current dir */
   if (!file_exists ("./INDEX")) 
      fetch_index_file ();
            
   if (!(fp = fopen("./INDEX", "r"))) {
      perror ("fopen");
      exit (1);
   }
   
   str_count = pattern_count = 0;
   
   /* make sure we'll read one whole line from INDEX */
   while (!feof(fp) && fgets(bigbuf, 100000, fp)) {
      
      /* remove CR/LF from the end of the string */
      bigbuf[strlen(bigbuf) - 1] = '\0';
      
      /* create the head of the queue and explode the string */
      explode_push_queue (bigbuf, &expl_head);
      expl_tail = expl_head;
      str_count = explode ('|', bigbuf, &expl_tail);
      
      /* take only four strings from the exploded line:
       * package name, description, category and maintainer
       * for more info take a look at "explode.h"
       */
      fname = expl_head->next->str; /* package name */
         
      /* pass away two elements */
      expl_current = expl_head->next;
      for (i = 0; i < 3; i++, expl_current = expl_current->next) ;
       
      desc = expl_current->str; /* description of the package */
      
      expl_current = expl_current->next;
      maint = expl_current->next->str; /* maintainer of the package */
      fpath = expl_current->next->next->str; /* category of the package */
           
      /* if we have a matched pattern - save it */
      /* searching by description and package name */
      if (str_count && ((strstr(strtolower(expl_head->next->str), strtolower(pattern))) || \
            strstr(strtolower(desc), strtolower(pattern)))) {
                 
         /* save the matched pattern */        
         fnode_push (fnode_head, fname, fpath, desc, maint); 
         pattern_count++;
      }
      
      /* destroy the queue for the next loop */
      explode_destroy_queue (expl_head);
   }
   
   fclose (fp);
   
   return pattern_count;
}

/* builds a complete URL to the ftp package root path */
char *build_ftp_path (BUILDFTPARGS)
{
   char *ftppath, buf[MAXPATHLEN + 1];
   char *machtype, *packageroot, *osrelease;
   
   /* checking for environment variables */
   if (!(packageroot = getenv("PACKAGEROOT"))) {
      fprintf (stdout, "==>> NOTICE: PACKAGEROOT was not defined..\n");
      fprintf (stdout, "==>> Using default PACKAGEROOT - %s\n\n", DEFAULTPACKAGEROOT);
      packageroot = (char *) malloc (strlen(DEFAULTPACKAGEROOT) + 1);
      strcpy (packageroot, DEFAULTPACKAGEROOT);
   }
   
   if (!(machtype = getenv("MACHTYPE"))) {
      fprintf (stdout, "==>> NOTICE: MACHTYPE was not defined..\n");
      fprintf (stdout, "==>> Using default MACHTYPE - %s\n\n", DEFAULTMACHTYPE);
      machtype = (char *) malloc (strlen(DEFAULTMACHTYPE) + 1);
      strcpy (machtype, DEFAULTMACHTYPE);
   }
   
   if (!(osrelease = getenv("OSRELEASE"))) {
      fprintf (stdout, "==>> NOTICE: OSRELEASE was not defined..\n");
      fprintf (stdout, "==>> Using default OSRELEASE - %s\n\n", DEFAULTOSRELEASE);
      osrelease = (char *) malloc (strlen(DEFAULTOSRELEASE) + 1);
      strcpy (osrelease, DEFAULTOSRELEASE);
   }
   
   snprintf (buf, sizeof(buf), "%s/pub/FreeBSD/ports/%s/packages-%s/", packageroot, machtype, osrelease);
   ftppath = (char *) malloc (strlen(buf) + 1);
   if (!ftppath) {
      perror ("malloc");
      exit (1);
   }
   ftppath = buf;
   
   return ftppath;
}
